use std::sync::Arc;

use wasm_bindgen_futures::{spawn_local, JsFuture};
use web_sys::{console, wasm_bindgen::UnwrapThrowExt, window, Response};
use super::{Environment, Handle, Rader};

#[derive(Clone)]
struct Fetcher {
    next: Option<Arc<Fetcher>>,
    filename: String,
    previous_text: String,
    rader: Rader,
    handle: Handle
}

pub fn cat(mut env: Environment) {
    if env.arguments.len() == 1 {
        env.rader.push("cat: no filename provided\n".to_string());
        env.handle.set(env.rader);
        return;
    }
    
    let mut previous_fetcher = None;
    for filename in env.arguments[1..].iter().rev() {
        let next_fetcher = Fetcher {
            next: previous_fetcher,
            filename: filename.to_string(),
            previous_text: String::new(),
            rader: env.rader.clone(),
            handle: env.handle.clone(),
        };
        previous_fetcher = Some(Arc::new(next_fetcher));
    }

    spawn_local(fetch((*previous_fetcher.unwrap_throw()).clone()));

}

async fn fetch(mut fetcher: Fetcher) {
    let text = fetch_single_file(fetcher.filename).await;

    let text = fetcher.previous_text + &text;

    if let Some(fetcher) = fetcher.next {
        let mut fetcher = (*fetcher).clone();
        fetcher.previous_text = text;
        Box::pin(fetch(fetcher.clone())).await
    } else {
        fetcher.rader.push(text.clone());
        fetcher.handle.set(fetcher.rader);
    }
}

async fn fetch_single_file(filename: String) -> String {
    let resource = format!("/fs/{filename}");

    let window = window().unwrap_throw();

    console::log_1(&"Got Window".into());

    let future: JsFuture = window.fetch_with_str(&resource).into();
    let response = future.await.unwrap_throw();
    let response = Response::from(response);

    if response.status() == 404 {
        return format!("cat: {filename}: No such file or directory\n");
    }

    console::log_1(&"Got Response".into());

    let text: JsFuture = response.text().unwrap_throw().into();
    let text = text.await.unwrap_throw();
    let text = text.as_string().unwrap_throw();

    console::log_1(&"Got Text".into());
    
    text
}
