use log::{error, info};
use yew::UseStateHandle;

mod clear;
mod cat;
mod echo;

pub type Rader = Vec<String>;
pub type Handle = UseStateHandle<Rader>;

struct Environment<'a> {
    pub rader: Rader,
    pub handle: Handle,
    pub arguments: Vec<&'a str>,
}

pub fn run(command: String, rader: Rader, handle: Handle) {
    let arguments = command.trim().split_whitespace().collect::<Vec<_>>();

    info!("{:?}", arguments);

    if arguments.len() == 0 {
        handle.set(rader);
        return;
    }

    let environment = Environment {
        rader,
        handle,
        arguments,
    };

    match environment.arguments[0] {
        "clear" => clear::clear(environment),
        "cat" => cat::cat(environment),
        "echo" => echo::echo(environment),
        _ => unrecognized_command(environment),
    }
}

fn unrecognized_command(mut env: Environment) {
    env.rader.push(format!("vsh: {}: command not found\n", env.arguments[0]));
    env.handle.set(env.rader);
    error!("Unrecognized Command");
}
